@@ -10,10 +10,12 @@ use serde::{Deserialize, Serialize};
 #[allow(non_camel_case_types, clippy::upper_case_acronyms)]
 pub enum TransformFn {
     NONE,
-    /// The SRGB "gamma compensation" function
-    sRGB_Gamma,
+    /// The sRGB transfer functions (aka "gamma correction")
+    sRGB,
     /// Oklab conversion from xyz
     Oklab,
+    /// Oklch (Oklab's LCh variant) conversion from xyz
+    Oklch,
     /// CIE xyY transform
     CIE_xyY,
     /// CIELAB transform
@@ -41,6 +43,11 @@ pub enum TransformFn {
     ICtCp_PQ,
     /// BT.2100 ICtCp with HLG transfer function
     ICtCp_HLG,
+    /// The BT.601/BT.709/BT.2020 (they are equivalent) OETF and inverse.
+    BT_601,
+    /// SMPTE ST 2084:2014 aka "Perceptual Quantizer" transfer functions used in BT.2100
+    /// for digitally created/distributed HDR content.
+    PQ,
     // ACEScc is a logarithmic transform
     // ACES_CC,
     // ACEScct is a logarithmic transform with toe
@@ -63,6 +70,12 @@ pub enum RGBPrimaries {
     BT_2020,
     AP0,
     AP1,
+    /// P3 is the primaries for DCI-P3 and the variations with different white points
+    P3,
+    ADOBE_1998,
+    ADOBE_WIDE,
+    APPLE,
+    PRO_PHOTO,
     CIE_RGB,
     /// The reference XYZ color space
     CIE_XYZ,
@@ -76,6 +89,15 @@ impl RGBPrimaries {
             Self::BT_2020 => &[[0.708, 0.292], [0.17, 0.797], [0.131, 0.046]],
             Self::AP0 => &[[0.7347, 0.2653], [0.0000, 1.0000], [0.0001, -0.0770]],
             Self::AP1 => &[[0.713, 0.293], [0.165, 0.830], [0.128, 0.044]],
+            Self::ADOBE_1998 => &[[0.64, 0.33], [0.21, 0.71], [0.15, 0.06]],
+            Self::ADOBE_WIDE => &[[0.735, 0.265], [0.115, 0.826], [0.157, 0.018]],
+            Self::PRO_PHOTO => &[
+                [0.734699, 0.265301],
+                [0.159597, 0.840403],
+                [0.036598, 0.000105],
+            ],
+            Self::APPLE => &[[0.625, 0.34], [0.28, 0.595], [0.155, 0.07]],
+            Self::P3 => &[[0.680, 0.320], [0.265, 0.690], [0.150, 0.060]],
             Self::CIE_RGB => &[[0.7350, 0.2650], [0.2740, 0.7170], [0.1670, 0.0090]],
             Self::CIE_XYZ => &[[1.0, 0.0], [0.0, 1.0], [0.0, 0.0]],
         }
@@ -108,6 +130,8 @@ pub enum WhitePoint {
     D65,
     /// North sky daylight
     D75,
+    /// P3-DCI white point, sort of greenish
+    P3_DCI,
     /// Cool fluorescent
     F2,
     /// Daylight fluorescent, D65 simulator
@@ -128,6 +152,7 @@ impl WhitePoint {
             Self::D60 => &[0.9523, 1.00000, 1.00859],
             Self::D65 => &[0.95047, 1.00000, 1.08883],
             Self::D75 => &[0.94972, 1.00000, 1.22638],
+            Self::P3_DCI => &[0.89458689458, 1.00000, 0.95441595441],
             Self::E => &[1.00000, 1.00000, 1.00000],
             Self::F2 => &[0.99186, 1.00000, 0.67393],
             Self::F7 => &[0.95041, 1.00000, 1.08747],
@@ -136,12 +161,14 @@ impl WhitePoint {
     }
 }
 
+/// A color space defined in data by its [Primaries][RGBPrimaries], [white point][WhitePoint], and an optional [invertible transform function][TransformFn].
+///
 /// See [spaces][crate::spaces] for defined color spaces.
 ///
 /// [ColorSpace] assumes that a color space is one of
 /// - the CIE XYZ color space
 /// - an RGB color space
-/// - an invertible mapping from one the above ([TransformFn])
+/// - a color space which may be defined as an invertible mapping from one the above ([TransformFn])
 ///
 /// An example of a [TransformFn] is the sRGB "opto-eletronic transfer function", or
 /// "gamma compensation".
@@ -154,7 +181,8 @@ impl WhitePoint {
 /// A linear RGB [ColorSpace] can be thought of as defining a relative coordinate system in the CIE XYZ
 /// color coordinate space, where three RGB primaries each define an axis pointing from
 /// the black point (0,0,0) in CIE XYZ.
-/// Non-linear [ColorSpace]s - such as sRGB with gamma compensation applied - are defined as a mapping from a linear
+///
+/// Non-linear [ColorSpace]s - such as sRGB with gamma compensation applied - are defined as a non-linear mapping from a linear
 /// [ColorSpace]'s coordinate system.
 #[derive(Debug, Copy, Clone, PartialEq, Hash, Eq)]
 #[cfg_attr(feature = "serde1", derive(Serialize, Deserialize))]
@@ -261,14 +289,23 @@ pub mod color_spaces {
     /// with a [D65 whitepoint.][WhitePoint::D65]
     /// Linear sRGB is equivalent to [BT_709].
     pub const LINEAR_SRGB: ColorSpace = ColorSpace::linear(RGBPrimaries::BT_709, WhitePoint::D65);
-    /// BT_709 is a linear encoding in [BT.709 primaries][RGBPrimaries::BT_709]
-    /// with a [D65 whitepoint.][WhitePoint::D65]. It's equivalent to [Linear sRGB][LINEAR_SRGB]
-    pub const BT_709: ColorSpace = ColorSpace::linear(RGBPrimaries::BT_709, WhitePoint::D65);
-    /// sRGB is [Linear sRGB][LINEAR_SRGB] with the sRGB tone response curve applied, also called "gamma-compressed".
-    pub const SRGB: ColorSpace = ColorSpace::new(
+
+    /// Encoded sRGB is [Linear sRGB][LINEAR_SRGB] with the [sRGB OETF](TransformFn::sRGB) applied (also called "gamma-compressed").
+    pub const ENCODED_SRGB: ColorSpace = ColorSpace::new(
         RGBPrimaries::BT_709,
         WhitePoint::D65,
-        TransformFn::sRGB_Gamma,
+        TransformFn::sRGB,
+    );
+
+    /// BT.709 is a linear encoding in [BT.709 primaries][RGBPrimaries::BT_709]
+    /// with a [D65 whitepoint.][WhitePoint::D65]. It's equivalent to [Linear sRGB][LINEAR_SRGB]
+    pub const BT_709: ColorSpace = ColorSpace::linear(RGBPrimaries::BT_709, WhitePoint::D65);
+
+    /// Encoded BT.709 is [BT.709](BT_709) with the [BT.709 OETF](TransformFn::BT_601) applied.
+    pub const ENCODED_BT_709: ColorSpace = ColorSpace::new(
+        RGBPrimaries::BT_709,
+        WhitePoint::D65,
+        TransformFn::BT_601,
     );
 
     /// ACEScg is a linear encoding in [AP1 primaries][RGBPrimaries::AP1]
@@ -282,15 +319,56 @@ pub mod color_spaces {
     /// with white point [E][WhitePoint::E].
     pub const CIE_RGB: ColorSpace = ColorSpace::linear(RGBPrimaries::CIE_RGB, WhitePoint::E);
 
+    /// CIE XYZ reference color space. Uses [CIE XYZ primaries][RGBPrimaries::CIE_XYZ]
+    /// with white point [D65][WhitePoint::D65].
+    pub const CIE_XYZ: ColorSpace = ColorSpace::linear(RGBPrimaries::CIE_XYZ, WhitePoint::D65);
+
     /// BT.2020 is a linear encoding in [BT.2020 primaries][RGBPrimaries::BT_2020]
     /// with a [D65 white point][WhitePoint::D65]
-    /// BT.2100 has the same color space as BT.2020.
+    /// BT.2100 has the same linear color space as BT.2020.
     pub const BT_2020: ColorSpace = ColorSpace::linear(RGBPrimaries::BT_2020, WhitePoint::D65);
 
-    /// Oklab is a non-linear encoding in [XYZ][RGBPrimaries::CIE_XYZ],
-    /// with a [D65 whitepoint][WhitePoint::D65]
-    pub const OKLAB: ColorSpace =
-        ColorSpace::new(RGBPrimaries::CIE_XYZ, WhitePoint::D65, TransformFn::Oklab);
+    /// Encoded BT.2020 is [BT.2020](BT_2020) with the [BT.2020 OETF][TransformFn::BT_601] applied.
+    pub const ENCODED_BT_2020: ColorSpace = ColorSpace::new(
+        RGBPrimaries::BT_2020,
+        WhitePoint::D65,
+        TransformFn::BT_601,
+    );
+
+    /// Encoded BT.2100 PQ is [BT.2020](BT_2020) (equivalent to the linear BT.2100 space) with
+    /// the [Perceptual Quantizer inverse EOTF][TransformFn::PQ] applied.
+    pub const ENCODED_BT_2100_PQ: ColorSpace = ColorSpace::new(
+        RGBPrimaries::BT_2020,
+        WhitePoint::D65,
+        TransformFn::PQ,
+    );
+
+    /// Oklab is a non-linear, perceptual encoding in [XYZ][RGBPrimaries::CIE_XYZ],
+    /// with a [D65 whitepoint][WhitePoint::D65].
+    ///
+    /// Oklab's perceptual qualities make it a very attractive color space for performing
+    /// blend operations between two colors which you want to be perceptually pleasing.
+    /// See [this article](https://bottosson.github.io/posts/oklab/)
+    /// for more on why you might want to use the Oklab colorspace.
+    pub const OKLAB: ColorSpace = ColorSpace::new(
+        RGBPrimaries::CIE_XYZ,
+        WhitePoint::D65,
+        TransformFn::Oklab,
+    );
+
+    /// Oklch is a non-linear, perceptual encoding in [XYZ][RGBPrimaries::CIE_XYZ],
+    /// with a [D65 whitepoint][WhitePoint::D65]. It is a variant of [Oklab][OKLAB]
+    /// with LCh coordinates intead of Lab.
+    ///
+    /// Oklch's qualities make it a very attractive color space for performing
+    /// computational modifications to a color. You can think of it as an improved
+    /// version of an HSL/HSV-style color space. See [this article](https://bottosson.github.io/posts/oklab/)
+    /// for more on why you might want to use the Oklch colorspace.
+    pub const OKLCH: ColorSpace = ColorSpace::new(
+        RGBPrimaries::CIE_XYZ,
+        WhitePoint::D65,
+        TransformFn::Oklch,
+    );
 
     /// ICtCp_PQ is a non-linear encoding in [BT.2020 primaries][RGBPrimaries::BT_2020],
     /// with a [D65 whitepoint][WhitePoint::D65], using the PQ transfer function
@@ -307,17 +385,67 @@ pub mod color_spaces {
         TransformFn::ICtCp_HLG,
     );
 
+    /// Encoded Display P3 is [Display P3][DISPLAY_P3] with the [sRGB OETF](TransformFn::sRGB) applied.
+    pub const ENCODED_DISPLAY_P3: ColorSpace = ColorSpace::new(
+        RGBPrimaries::P3,
+        WhitePoint::D65,
+        TransformFn::sRGB,
+    );
+
+    /// Display P3 by Apple is a linear encoding in [P3 primaries][RGBPrimaries::P3]
+    /// with a [D65 white point][WhitePoint::D65]
+    pub const DISPLAY_P3: ColorSpace = ColorSpace::linear(RGBPrimaries::P3, WhitePoint::D65);
+
+    /// P3-D60 (ACES Cinema) is a linear encoding in [P3 primaries][RGBPrimaries::P3]
+    /// with a [D60 white point][WhitePoint::D60]
+    pub const P3_D60: ColorSpace = ColorSpace::linear(RGBPrimaries::P3, WhitePoint::D60);
+
+    /// P3-DCI (Theater) is a linear encoding in [P3 primaries][RGBPrimaries::P3]
+    /// with a [P3-DCI white point][WhitePoint::P3_DCI]
+    pub const P3_THEATER: ColorSpace = ColorSpace::linear(RGBPrimaries::P3, WhitePoint::P3_DCI);
+
+    /// Adobe RGB (1998) is a linear encoding in [Adobe 1998 primaries][RGBPrimaries::ADOBE_1998]
+    /// with a [D65 white point][WhitePoint::D65]
+    pub const ADOBE_1998: ColorSpace =
+        ColorSpace::linear(RGBPrimaries::ADOBE_1998, WhitePoint::D65);
+
+    /// Adobe Wide Gamut RGB is a linear encoding in [Adobe Wide primaries][RGBPrimaries::ADOBE_WIDE]
+    /// with a [D50 white point][WhitePoint::D50]
+    pub const ADOBE_WIDE: ColorSpace =
+        ColorSpace::linear(RGBPrimaries::ADOBE_WIDE, WhitePoint::D50);
+
+    /// Pro Photo RGB is a linear encoding in [Pro Photo primaries][RGBPrimaries::PRO_PHOTO]
+    /// with a [D50 white point][WhitePoint::D50]
+    pub const PRO_PHOTO: ColorSpace = ColorSpace::linear(RGBPrimaries::PRO_PHOTO, WhitePoint::D50);
+
+    /// Apple RGB is a linear encoding in [Apple primaries][RGBPrimaries::APPLE]
+    /// with a [D65 white point][WhitePoint::D65]
+    pub const APPLE: ColorSpace = ColorSpace::linear(RGBPrimaries::APPLE, WhitePoint::D65);
+
     /// Array containing all built-in color spaces.
-    pub const ALL_COLOR_SPACES: [ColorSpace; 9] = [
+    pub const ALL_COLOR_SPACES: [ColorSpace; 22] = [
+        color_spaces::LINEAR_SRGB,
+        color_spaces::ENCODED_SRGB,
         color_spaces::BT_709,
+        color_spaces::ENCODED_BT_709,
         color_spaces::BT_2020,
-        color_spaces::SRGB,
+        color_spaces::ENCODED_BT_2020,
+        color_spaces::ENCODED_BT_2100_PQ,
         color_spaces::ACES_CG,
         color_spaces::ACES2065_1,
         color_spaces::CIE_RGB,
+        color_spaces::CIE_XYZ,
         color_spaces::OKLAB,
         color_spaces::ICtCp_PQ,
         color_spaces::ICtCp_HLG,
+        color_spaces::PRO_PHOTO,
+        color_spaces::APPLE,
+        color_spaces::P3_D60,
+        color_spaces::P3_THEATER,
+        color_spaces::DISPLAY_P3,
+        color_spaces::ENCODED_DISPLAY_P3,
+        color_spaces::ADOBE_1998,
+        color_spaces::ADOBE_WIDE,
     ];
 }
 
@@ -338,11 +466,12 @@ impl Color {
     pub fn space(&self) -> ColorSpace {
         self.space
     }
-    /// Equivalent to `Color::new(x, y, z, kolor::spaces::SRGB)`
+
+    /// Equivalent to `Color::new(x, y, z, kolor::spaces::ENCODED_SRGB)`
     pub fn srgb(x: FType, y: FType, z: FType) -> Self {
         Self {
             value: Vec3::new(x, y, z),
-            space: color_spaces::SRGB,
+            space: color_spaces::ENCODED_SRGB,
         }
     }
 
@@ -405,8 +534,8 @@ mod test {
 
     #[test]
     fn linear_srgb_to_srgb() {
-        let transform = ColorTransform::new(TransformFn::NONE, TransformFn::sRGB_Gamma).unwrap();
-        let mut test = Vec3::new(0.35, 0.1, 0.8);
+        let transform = ColorTransform::new(TransformFn::NONE, TransformFn::sRGB).unwrap();
+        let test = Vec3::new(0.35, 0.1, 0.8);
         let result = transform.apply(test, WhitePoint::D65);
         assert_eq!(result, Vec3::new(0.6262097, 0.34919018, 0.9063317));
     }
@@ -425,7 +554,7 @@ mod test {
 
     #[test]
     fn aces_cg_to_srgb() {
-        let conversion = ColorConversion::new(spaces::ACES_CG, spaces::SRGB);
+        let conversion = ColorConversion::new(spaces::ACES_CG, spaces::ENCODED_SRGB);
         let result = conversion.convert(Vec3::new(0.35, 0.1, 0.8));
         assert_eq!(result, Vec3::new(0.46201152, 0.06078783, 0.8996733));
     }
@@ -497,7 +626,7 @@ mod test {
 
     #[test]
     fn cielab_test() {
-        let srgb = Color::new(1.0, 0.5, 0.0, spaces::SRGB);
+        let srgb = Color::new(1.0, 0.5, 0.0, spaces::ENCODED_SRGB);
         let cielab = srgb.to(srgb.space.to_cielab());
         let cielab_inverse = cielab.to(srgb.space);
         let cielch = srgb.to(srgb.space.to_cielch());
@@ -527,7 +656,7 @@ mod test {
 
     #[test]
     fn cie_uvV_test() {
-        let srgb = Color::new(1.0, 0.5, 0.0, spaces::SRGB);
+        let srgb = Color::new(1.0, 0.5, 0.0, spaces::ENCODED_SRGB);
         let uvV = srgb.to(srgb.space.to_cielab());
         let uvV_inverse = uvV.to(srgb.space);
         println!("srgb {:?}", srgb.value);
